

#
# Event driver. Register a file descriptor and function to be invoked
# when input is ready.
#
# A timer is a list [trigger_time, func, arg, next_timer]. We keep a list
# of timers sorted by trigger time.
# 

package EventDriver;

my $nextfh = "EVENTFH0000";

my $NEXT = 4;

sub new
{
    my($class) = @_;
    my($self) = {};
    bless $self, $class;

    $self->{timers} = undef;
    $self->{fds} = {};
    $self->{fd_list} = [];
    $self->{nexus_fds} = [];
    
    return $self;
}

sub timer_lt
{
    my($t1, $t2) = @_;

    if ($$t1[0] < $$t2[0])
    {
	return 1;
    }
    else
    {	
	return $$t1[1] < $$t2[1];
    }
}

sub register_timer
{
    my($self, $ms, $func, $arg) = @_;
    my($timer, $trigger_time_s, $trigger_time_us);
    my ($last_t, $t);

    $trigger_time_s = time + int($ms / 1000);
    $trigger_time_us = int(1000 * ($ms % 1000));

    $timer = [$trigger_time_s, $trigger_time_us, $func, $arg, undef];

    undef $last_t;
    for ($t = $self->{timers}; defined($t) && &timer_lt($t, $timer); )
    {
	$last_t = $t;
	$t = $t->[$NEXT];
    }
    
    if (defined($last_t))
    {
	$last_t->[$NEXT] = $timer;
    }
    else
    {
	$self->{timers} = $timer;
    }
    $timer->[$NEXT] = $t;
}

sub dump
{
    my($self) = @_;
    my($t);

    print "ED $self dump:\n";
    for ($t = $self->{timers}; defined($t); $t = $t->[$NEXT])
    {
	print "$t: ", join(":", @$t), "\n";
    }
    
}

sub print_timer
{
    my($t) = @_;
    print "$t: ", join(":", @$t), "\n";
}

sub register_nexus
{
    my($self) = @_;

    Nexus::register_fd_notifiers(\&add_notifier, $self,
				 \&remove_notifier, $self);
}

sub add_notifier
{
#    print "add_notifier: <", join(":", @_), ">\n";
    my($self, $fh) = @_;

    $self->register($fh, \&do_nexus_poll);
}

sub remove_notifier
{
    my($self, $fh) = @_;

    $self->unregister($fh);
}

sub do_nexus_poll
{
    my($arg, $fh) = @_;
#    print "Nexus ready\n";
    Nexus::poll(0);
}

sub register
{
    my($self, $fh, $func, $arg) = @_;

#    print "Register $fh $func $arg\n";
    $self->{fds}->{$fh} = [$fh, $func, $arg];
    push(@{$self->{fd_list}}, $fh);
}

sub unregister
{
    my($self, $fh) = @_;
    my(@nfd);

#    print "Unregister $fh\n";
    delete $self->{fds}->{$fh};

    @nfd = ();
    for (@{$self->{fd_list}})
    {
	if ($_ ne $fh)
	{
	    push(@nfd, $_);
	}
    }
    @{$self->{fd_list}} = @nfd;
}

sub poll
{
    my($self) = @_;
    my($mask, $ready, $done, $n, $i, @fh);
    my($interval_s, $interval_us);

    $done = 0;
    
    while (!$done)
    {
	if (defined($timer = $self->{timers}))
	{
	    my($now_s) = time;
	    my($now_us) = 0;

#	    print "Timer: now=$now_s:now_us ";
#	    print_timer $timer;
	    
	    $interval_s = $timer->[0] - $now_s;
	    $interval_us = $timer->[1] - $now_us;
	    if ($interval_us < 0)
	    {
		$interval_s--;
		$interval_us += 1000000;
	    }
#	    print "Interval: $interval_s $interval_us\n";
	}
	
	$mask = '';
	@fh = @{$self->{fd_list}};
	for $handle (@fh)
	{
#	    print "Adding $handle\n";
	    vec($mask, fileno($handle), 1) = 1;
	}

	$n = select($ready = $mask, undef, undef,
		    defined($interval_s) 
		       ? $interval_s + $interval_us / 1000000
		       : undef);
		        
	
#	print "Select returns $n\n";
	if ($n < 0)
	{
	    next if ($! =~ /interrupted\s*system\s*call/i);
	    die "Select fails: $!\n";
	}
	elsif ($n == 0)
	{
	    $self->check_timers();
	    $done = 1;
	}
	else
	{
	    $done = 1;
	    for $fh (@fh)
	    {
		last if $n == 0;
		if (vec($ready, fileno($fh), 1) == 1)
		{
#		    print "$fh is ready\n";
		    $self->invoke($fh);
		    $n--;
		}
	    }
	}
    }
}

#
# See if we can fire any of the interval timers.
#
sub check_timers
{
    my($self) = @_;
    my($now_s,$now_us);
    my($timer);
    
    $now_s = time;
    $now_us = 0;

    while (1)
    {
	$timer = $self->{timers};
	last unless defined($timer);
	if ($timer->[0] < $now_s ||
	    (($timer->[0] == $now_s) && ($timer->[1] <= $now_us)))
	{
	    #
	    # And remove the timer from the list.
	    #
	    $self->{timers} = $timer->[$NEXT];

	    #
	    # And fire it off. Remove from the list *before* firing
	    # process in case the process adds another timer to the list.
	    #
#	    print "Fire timer $timer ", @$timer, "\n";
	    eval { &{$timer->[2]}($timer->[3]); };

	    if ($@)
	    {
		warn "Eval failed: $@";
	    }
	}
	else
	{
	    last;
	}
    }
}

sub invoke
{
    my($self, $fh) = @_;
    my($entry, $func, $arg);

    $entry = $self->{fds}->{$fh};
    if (defined($entry))
    {
	$func = $$entry[1];
	$arg = $$entry[2];
	$arg = '' unless $arg;
#	print "got func=$func arg=$arg for $fh\n";
	&$func($arg, $fh);
    }
    else
    {
	warn "Cannot find entry for $fh\n";
    }
}

1;
