#
#  Sockets package for Perl 5
#  Via Bob Olson
#  
# Documentation:
#   This is a class of filehandles, implemented to use BSD sockets.
# This enables socket communication for IPC.  The API is as follows:
#  
#   $foo = new Sock($protocol);
#
#  This creates a new instance of the socket class, with the specified 
#   protocol if the protocol specification is not specified, the socket 
#   is TCP by default.
#
#   $foo->connect($remote_host, $remote_port);
#
#  This causes the socket to attempt to connect to the specified Host/port.
#
#  $foo->setup_listener($port, $numq);
#
#  This makes the socket start listening on the port specified.  The 
#    $numq argument tells the socket how many connections to maintain
#    before it starts refusing connections.
#
#  $foo->accept();
#
#  This accepts a new connections on the socket, and returns the new
#    connection.
#
#  $foo->local_addr();
#
#  This returns the local address of the socket.
#
#  $foo->local_port();
#
#  This returns the local port of the socket.
#
#  $foo->local_family();
#
#  This returns the family of the local socket.
#
#  $foo->local_ip();
#
#  This returns the IP Address of the local socket.
#
#  $foo->local_hostname();
#
#  This returns the hostname of the local side of the socket.
#
#  $foo->remote_addr();
#
#  This returns the remote address of the socket.
#
#  $foo->remote_port();
#
#  This returns the remote port of the socket.
#
#  $foo->remote_family();
#
#  This returns the family of the remote socket.
#
#  $foo->remote_ip();
#
#  This returns the IP Address of the remote socket.
#
#  $foo->remote_hostname();
#
#  This returns the hostname of the remote side of the socket.
#
#  $foo->get_addr_for_host($host);
#
#  This gets the network address of the host specified.
#
#  $foo->read($size);
#
#  This function returns a buffer of $size, that it read from the socket.

package Sock;

require 'errno.ph';
require POSIX;

use Socket;

@ISA = qw(FileHandle);

$debug = $main'debug;

chop($local_hostname = `hostname`);
$sockaddr = 'S n a4 x8';
my $NextFH = "SOCKFH0000";

sub new
{
    my($class, $protoname) = @_;
    my($self) = *{"Sock::" . $NextFH++};
    my($name, $aliases, $proto);

#    print "self=$self\n";

    $protoname = 'tcp' unless $protoname ne '';
#    print "proto=$protoname\n";
    ($name, $aliases, $proto) = getprotobyname($protoname);
#    print "proto=$proto\n";
    if (!socket($self, PF_INET, SOCK_STREAM, $proto))
    {
	die "socket failed: $!\n";
    }
    bless \$self, $class;

}

sub connect
{
    my($self, $remote, $port) = @_;

    my($name, $aliases, $proto, $type, $len);
    my($remoteaddr, $localaddr, $hostname);
    my($remote_saddr, $local_saddr);
    
    ($name,$aliases,$port) = getservbyname($port, 'tcp')
	unless $port =~ /^\d+$/;

    $remoteaddr = &get_addr_for_host($remote);
    $localaddr = &get_addr_for_host($local_hostname);

    ($name,$aliases,$type,$len,$localaddr) = gethostbyname($local_hostname);

    $remote_saddr = pack($sockaddr, &PF_INET, $port, $remoteaddr);
    $local_saddr = pack($sockaddr, &PF_INET, $local_port, $localaddr);
    
    my($n) = 0;
   while (1) {
       if (bind($self, $local_saddr)) {
	   last;
       }
       if ($! eq "Address in use") {
#       if ($! == &EADDRINUSE) {
	    if ($n == 0) {
		print "Address in use, sleeping";
	    }
	    sleep(1);
	} else	{
	    die "bind: $!";
	}
    }
    connect($self, $remote_saddr) || die "$prog: connect: $!";
    
    select($self); $| = 1;
    select(STDOUT);
}
    
sub setup_listener
{
    my($self, $port, $listen_q) = @_;
    my($name, $aliases, $proto, $this);

    $listen_q = 5 if !$listen_q;

    ($name, $aliases, $port) = getservbyname($port, $protoname)
	unless $port =~ /^\d+$/;

    setsockopt($self, &SOL_SOCKET, &SO_REUSEADDR, 1) ||
	die "setsockopt SO_REUSEADDR failed: $!\n";

    $this = pack($sockaddr, &PF_INET, $port, "\0\0\0\0");
    
    my($n) = 0;

    while (1) {
	if (bind($self, $this))	{
	    last;
	}
	    
        if ($! eq "Address in use") {
#	if ($! == &EADDRINUSE) {
	    if ($n == 0) {
		print "Address in use, sleeping";
	    }
	    print "." if $n++ % 5 == 0;
	    
	    sleep(1);
	} else {
	    die "bind: $!";
	}
    }
	   
    listen($self, $listen_q) || die "listen: $!";
}

sub accept
{
    my($self) = @_;
    my($ns) = new Sock;

    if (!accept($ns, $self))  {
	die "accept failed: $!\n";
    }
    select((select($ns), $| = 1)[0]);
    $ns;
}

sub local_addr
{
    my($self) = shift;
    my($addr, $family, $port, $myAddr);

    if ($addr = getsockname($self))  {
	($family, $port, $myAddr) = unpack($sockaddr, $addr);
	return $myAddr;
    } else {
	return undef;
    }
}

sub local_port
{
    my($self) = shift;
    my($addr, $family, $port, $myAddr);

    if ($addr = getsockname($self)) {
	($family, $port, $myAddr) = unpack($sockaddr, $addr);
	return $port;
    } else {
	return undef;
    }
}

sub local_family
{
    my($self) = shift;
    my($addr, $family, $port, $myAddr);

    if ($addr = getsockname($self)) {
	($family, $port, $myAddr) = unpack($sockaddr, $addr);
	return $family;
    } else {
	return undef;
    }
}

sub local_ip
{
    my($self) = shift;
    my($addr);
    
    if ($addr = $self->local_addr) {
	return join(".", unpack('C4', $addr));
    } else {
	return undef;
    }
}

sub local_hostname
{
    my($self) = shift;
    my($addr);
    my($name,$aliases,$type,$len,$remoteaddr);
    
    if ($addr = $self->local_addr) {
	($name,$aliases,$type,$len,$remoteaddr) =
	    gethostbyaddr($addr, $self->local_family);
	return $name;
    } else  {
	return undef;
    }
}

sub remote_addr
{
    my($self) = shift;
    my($addr, $family, $port, $myAddr);

    if ($addr = getpeername($self)) {
	($family, $port, $myAddr) = unpack($sockaddr, $addr);
	return $myAddr;
    } else {
	return undef;
    }
}

sub remote_port
{
    my($self) = shift;
    my($addr, $family, $port, $myAddr);

    if ($addr = getpeername($self)) {
	($family, $port, $myAddr) = unpack($sockaddr, $addr);
	return $port;
    } else {
	return undef;
    }
}

sub remote_family
{
    my($self) = shift;
    my($addr, $family, $port, $myAddr);

    if ($addr = getpeername($self)) {
	($family, $port, $myAddr) = unpack($sockaddr, $addr);
	return $family;
    } else {
	return undef;
    }
}

sub remote_ip
{
    my($self) = shift;
    my($addr);
    
    if ($addr = $self->remote_addr) {
	return join(".", unpack('C4', $addr));
    } else {
	return undef;
    }
}

sub remote_hostname
{
    my($self) = shift;
    my($addr);
    my($name,$aliases,$type,$len,$remoteaddr);
    
    if ($addr = $self->remote_addr) {
	($name,$aliases,$type,$len,$remoteaddr) =
	    gethostbyaddr($addr, $self->remote_family);
	return $name;
    } else {
	return undef;
    }
}

sub get_addr_for_host
{
    local($remote) = @_;
    local($name, $aliases, $type, $len, $remoteaddr);
    
    ($name,$aliases,$type,$len,$remoteaddr) = gethostbyname($remote);
    if (!$name)  {
	if (@raddr = ($remote =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/)) {
	    ($name,$aliases,$type,$len,$remoteaddr) =
		gethostbyaddr(pack('C4', split(/\./, $remote)), &AF_INET);
	    if ($name) {
		print "Got name by gethostbyaddr: $name\n" if $verbose;;
	    } else  {
		print "Using packed address ", join('.', @raddr), "\n"
		    if $verbose;
		$remoteaddr = pack('C4', @raddr);
	    }
	} else {
	    die "Couldn't find host $remote\n";
	}
    } else  {
	print "got name $name\n" if $verbose;
    }
    print "found address ", join('.', unpack('C4', $remoteaddr)), "\n"
	if $verbose;
    $remoteaddr;
}

sub read
{
    local($self, $size) = @_;
    
    $retval = sysread($self, $retstring, $size);
    if (!defined($retval)) {
	return(undef);
    } else {
	return($retstring);
    }
}

sub read_line {
    local($self) = @_;
    $cmdlength = 80;
    $retval = sysread($self, $buffer, $cmdlength);

    return($retval, $buffer);
}

sub write {
    local($self, $line) = @_;
    $n = syswrite($self, $line, length($line));
    return $n;
}

1;
