/*-
 * Copyright (c) 1993-1994 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and the Network Research Group at
 *      Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <osfcn.h>
#include <stdio.h>
#include <string.h>
#include "inet.h"
#include "packet.h"
#include "reasm-jpeg.h"

/*
 * Initial size of both reassembly buffer.
 */
#define JPEG_BUFSIZE ( 16 * 1024 )

JpegReassembler::JpegReassembler()
	: jpeg_badoff_(0), jpeg_hugefrm_(0), decimate_(0), ndec_(0)
{
	rbsize_ = JPEG_BUFSIZE;
	rb0_.bp = new u_char[2 * JPEG_BUFSIZE];
	rb0_.ts = ~0;
	rb0_.drop = 0;
	rb1_.bp = &rb0_.bp[JPEG_BUFSIZE];
	rb1_.ts = ~0;
	rb1_.drop = 0;
	memset(slots_, 0, sizeof(slots_));
}

JpegReassembler::~JpegReassembler()
{
	delete rb0_.bp;
}

/*
 * Reassemble an RTP/JPEG stream.  Return a pointer to a buffer
 * each time we encounter an entire frame.  Otherwise, return 0.
 * Set len to the length of the jpeg data in the buffer.
 *
 * rh is the rtp header
 * vh is the start of the data in the rtp packet (jpeg header)
 * bp is the start of the data in the jpeg packet
 * len is the length of the jpeg data
 */
u_char* JpegReassembler::reassemble(rtphdr* rh, u_char* vh,
				    u_char* bp, int& len)
{
  jpeghdr* p = (jpeghdr*)vh;
  int off = (int)ntohl(p->off);
  int cc = len;
  if (off < 0) {
    ++jpeg_badoff_;
    return (0);
  }
  if (off + cc > rbsize_) {
    /*
     * Check for outrageous frame size.
     */
    if (off + cc > 250*1024) {
      ++jpeg_hugefrm_;
      return (0);
    }
    /*
     * Grow reassembly buffers.
     */
    int nsize = rbsize_;
    do {
      nsize <<= 1;
    } while (off + cc > nsize);
    u_char* p = new u_char[2 * nsize];
    memcpy(p, rb0_.bp, rbsize_);
    memcpy(p + nsize, rb1_.bp, rbsize_);
    delete rb0_.bp;
    rb0_.bp = p;
    rb1_.bp = p + nsize;
    rbsize_ = nsize;
  }
  /*
   * Initialize the slot data structure.
   */
  int seqno = ntohs(rh->rh_seqno);
  int s = seqno & JPEG_SLOTMASK;
  n_long ts = ntohl(rh->rh_ts);
  slots_[s].seqno = seqno;
  slots_[s].off = off;
  slots_[s].ts = ts;
  /*
   * Figure out which reassembly-buffer to use.  If we're not
   * already reassembling this frame, take over the older buffer.
   */
  rbuf* rb;
  if (ts == rb0_.ts)
    rb = &rb0_;
  else if (ts == rb1_.ts)
    rb = &rb1_;
  else {
    rb = ((int)(rb0_.ts - rb1_.ts) < 0) ? &rb0_ : &rb1_;
    rb->ts = ts;
    rb->drop = 0;
    /*
     * If we're decimating frames (to save cycles),
     * remember that we might want to drop the rest
     * of the packets from this frame.
     */
    if (decimate_) {
      if (--ndec_ <= 0)
	ndec_ = decimate_;
      else
	rb->drop = 1;
    }
  }
  if (rb->drop) {
    return (0);
  }

  memcpy((char*)&rb->bp[off], (char*)bp, cc);

  /*
   * Check if we're at end-of-frame.  If not, see if we're
   * filling a hole.  If not, return.  Otherwise, drop out
   * below and check for an entire frame.  We set cc to be
   * the entire frame size in the if-else below.
   */
  if ((ntohs(rh->rh_flags) & RTP_M) != 0) {
    slots_[s].eof = cc;
    cc += off;
  } else {
    slots_[s].eof = 0;
    int ns = s;
    do {
      ns = (ns + 1) & JPEG_SLOTMASK;
      if (slots_[ns].ts != ts || ns == s) {
	return (0);
      }
    } while (slots_[ns].eof != 0);
    cc = int(slots_[ns].eof + slots_[ns].off);
  }
  /*
   * At this point, we know we have an end-of-frame, and
   * all packets from slot 's' up until the end-of-frame.
   * Scan backward from slot 's' making sure we have all
   * packets from the start-of-frame (off == 0) to 's'.
   */
  int ps = s;
  do {
    ps = (ps - 1) & JPEG_SLOTMASK;
    if (slots_[ps].ts != ts || ps == s) {
      return (0);
    }
  } while (slots_[ps].off != 0);

  len = cc;
  return (rb->bp);
}
